@@ -9,6 +9,7 @@ import sys
 
 from .. import config
 from .. import errors
+from .. import logging
 from ..utils import queue as _queue
 
 
@@ -28,10 +29,15 @@ class Job(object):
         self.hooks = hooks
         self.report = report
 
-    def __call__(self, queue, interrupt_flag):
+    def __call__(self, queue, interrupt_flag, teardown_flag=None):
         """
         Continuously executes tests from 'queue' and records their
         details in 'report'.
+
+        If 'teardown_flag' is not None, then 'self.fixture.teardown()'
+        will be called before this method returns. If an error occurs
+        while destroying the fixture, then the 'teardown_flag' will be
+        set.
         """
 
         should_stop = False
@@ -52,6 +58,15 @@ class Job(object):
             # Drain the queue to unblock the main thread.
             Job._drain_queue(queue)
 
+        if teardown_flag is not None:
+            try:
+                if not self.fixture.teardown(finished=True):
+                    self.logger.warn("Teardown of %s was not successful.", self.fixture)
+                    teardown_flag.set()
+            except:
+                self.logger.exception("Encountered an error while tearing down %s.", self.fixture)
+                teardown_flag.set()
+
     def _run(self, queue, interrupt_flag):
         """
         Calls the before/after suite hooks and continuously executes
@@ -84,7 +99,7 @@ class Job(object):
 
         test(self.report)
         if config.FAIL_FAST and not self.report.wasSuccessful():
-            test.logger.info("%s failed, so stopping..." % (test.shortDescription()))
+            self.logger.info("%s failed, so stopping..." % (test.shortDescription()))
             raise errors.StopExecution("%s failed" % (test.shortDescription()))
 
         if not self.fixture.is_running():
