@@ -165,7 +165,6 @@ struct HostNotIn {
 
 /**
  * Replica set refresh period on the task executor.
- * This value must be equal to ReadPreferenceSettings::kMinimalMaxStalenessValue / 2
  */
 const Seconds kRefreshPeriod(30);
 }  // namespace
@@ -186,7 +185,6 @@ ReplicaSetMonitor::ReplicaSetMonitor(const MongoURI& uri)
 void ReplicaSetMonitor::init() {
     stdx::lock_guard<stdx::mutex> lk(_mutex);
     invariant(_executor);
-    invariant(kRefreshPeriod * 2 == ReadPreferenceSetting::kMinimalMaxStalenessValue);
     std::weak_ptr<ReplicaSetMonitor> that(shared_from_this());
     auto status = _executor->scheduleWork([=](const CallbackArgs& cbArgs) {
         if (auto ptr = that.lock()) {
@@ -1030,12 +1028,12 @@ HostAndPort SetState::getMatchingHost(const ReadPreferenceSetting& criteria) con
             if (!out.empty())
                 return out;
             return getMatchingHost(ReadPreferenceSetting(
-                ReadPreference::SecondaryOnly, criteria.tags, criteria.maxStalenessMS));
+                ReadPreference::SecondaryOnly, criteria.tags, criteria.maxStalenessSeconds));
         }
 
         case ReadPreference::SecondaryPreferred: {
             HostAndPort out = getMatchingHost(ReadPreferenceSetting(
-                ReadPreference::SecondaryOnly, criteria.tags, criteria.maxStalenessMS));
+                ReadPreference::SecondaryOnly, criteria.tags, criteria.maxStalenessSeconds));
             if (!out.empty())
                 return out;
             // NOTE: the spec says we should use the primary even if tags don't match
@@ -1058,7 +1056,7 @@ HostAndPort SetState::getMatchingHost(const ReadPreferenceSetting& criteria) con
                 return true;
             };
             // build comparator
-            if (criteria.maxStalenessMS.count()) {
+            if (criteria.maxStalenessSeconds.count()) {
                 auto masterIt = std::find_if(nodes.begin(), nodes.end(), isMaster);
                 if (masterIt == nodes.end() || !masterIt->lastWriteDate.toMillisSinceEpoch()) {
                     auto writeDateCmp = [](const Node* a, const Node* b) -> bool {
@@ -1078,18 +1076,19 @@ HostAndPort SetState::getMatchingHost(const ReadPreferenceSetting& criteria) con
                     } else {
                         Date_t maxWriteTime = (*latestSecNode)->lastWriteDate;
                         matchNode = [=](const Node& node) -> bool {
-                            return Milliseconds(maxWriteTime - node.lastWriteDate) +
+                            return duration_cast<Seconds>(maxWriteTime - node.lastWriteDate) +
                                 kRefreshPeriod <=
-                                criteria.maxStalenessMS;
+                                criteria.maxStalenessSeconds;
                         };
                     }
                 } else {
-                    Milliseconds primaryStaleness =
-                        masterIt->lastWriteDateUpdateTime - masterIt->lastWriteDate;
+                    Seconds primaryStaleness = duration_cast<Seconds>(
+                        masterIt->lastWriteDateUpdateTime - masterIt->lastWriteDate);
                     matchNode = [=](const Node& node) -> bool {
-                        return Milliseconds(node.lastWriteDateUpdateTime - node.lastWriteDate) -
+                        return duration_cast<Seconds>(node.lastWriteDateUpdateTime -
+                                                      node.lastWriteDate) -
                             primaryStaleness + kRefreshPeriod <=
-                            criteria.maxStalenessMS;
+                            criteria.maxStalenessSeconds;
                     };
                 }
             }
