@@ -95,14 +95,14 @@ StatusWithMatchExpression MatchExpressionParser::_parseSubField(const BSONObj& c
                                                                 const char* name,
                                                                 const BSONElement& e,
                                                                 const CollatorInterface* collator,
-                                                                int level) {
+                                                                bool topLevel) {
     // TODO: these should move to getGtLtOp, or its replacement
 
     if (mongoutils::str::equals("$eq", e.fieldName()))
         return _parseComparison(name, new EqualityMatchExpression(), e, collator);
 
     if (mongoutils::str::equals("$not", e.fieldName())) {
-        return _parseNot(name, e, collator, level);
+        return _parseNot(name, e, collator, topLevel);
     }
 
     int x = e.getGtLtOp(-1);
@@ -247,10 +247,10 @@ StatusWithMatchExpression MatchExpressionParser::_parseSubField(const BSONObj& c
         }
 
         case BSONObj::opELEM_MATCH:
-            return _parseElemMatch(name, e, collator, level);
+            return _parseElemMatch(name, e, collator, topLevel);
 
         case BSONObj::opALL:
-            return _parseAll(name, e, collator, level);
+            return _parseAll(name, e, collator, topLevel);
 
         case BSONObj::opWITHIN:
         case BSONObj::opGEO_INTERSECTS:
@@ -285,19 +285,10 @@ StatusWithMatchExpression MatchExpressionParser::_parseSubField(const BSONObj& c
 
 StatusWithMatchExpression MatchExpressionParser::_parse(const BSONObj& obj,
                                                         const CollatorInterface* collator,
-                                                        int level) {
-    if (level > kMaximumTreeDepth) {
-        mongoutils::str::stream ss;
-        ss << "exceeded maximum query tree depth of " << kMaximumTreeDepth << " at "
-           << obj.toString();
-        return {Status(ErrorCodes::BadValue, ss)};
-    }
-
+                                                        bool topLevel) {
     std::unique_ptr<AndMatchExpression> root = stdx::make_unique<AndMatchExpression>();
 
-    bool topLevel = (level == 0);
-    level++;
-
+    const bool childIsTopLevel = false;
     BSONObjIterator i(obj);
     while (i.more()) {
         BSONElement e = i.next();
@@ -309,7 +300,7 @@ StatusWithMatchExpression MatchExpressionParser::_parse(const BSONObj& obj,
                 if (e.type() != Array)
                     return {Status(ErrorCodes::BadValue, "$or must be an array")};
                 std::unique_ptr<OrMatchExpression> temp = stdx::make_unique<OrMatchExpression>();
-                Status s = _parseTreeList(e.Obj(), temp.get(), collator, level);
+                Status s = _parseTreeList(e.Obj(), temp.get(), collator, childIsTopLevel);
                 if (!s.isOK())
                     return s;
                 root->add(temp.release());
@@ -317,7 +308,7 @@ StatusWithMatchExpression MatchExpressionParser::_parse(const BSONObj& obj,
                 if (e.type() != Array)
                     return {Status(ErrorCodes::BadValue, "$and must be an array")};
                 std::unique_ptr<AndMatchExpression> temp = stdx::make_unique<AndMatchExpression>();
-                Status s = _parseTreeList(e.Obj(), temp.get(), collator, level);
+                Status s = _parseTreeList(e.Obj(), temp.get(), collator, childIsTopLevel);
                 if (!s.isOK())
                     return s;
                 root->add(temp.release());
@@ -325,7 +316,7 @@ StatusWithMatchExpression MatchExpressionParser::_parse(const BSONObj& obj,
                 if (e.type() != Array)
                     return {Status(ErrorCodes::BadValue, "$nor must be an array")};
                 std::unique_ptr<NorMatchExpression> temp = stdx::make_unique<NorMatchExpression>();
-                Status s = _parseTreeList(e.Obj(), temp.get(), collator, level);
+                Status s = _parseTreeList(e.Obj(), temp.get(), collator, childIsTopLevel);
                 if (!s.isOK())
                     return s;
                 root->add(temp.release());
@@ -370,7 +361,7 @@ StatusWithMatchExpression MatchExpressionParser::_parse(const BSONObj& obj,
         }
 
         if (_isExpressionDocument(e, false)) {
-            Status s = _parseSub(e.fieldName(), e.Obj(), root.get(), collator, level);
+            Status s = _parseSub(e.fieldName(), e.Obj(), root.get(), collator, childIsTopLevel);
             if (!s.isOK())
                 return s;
             continue;
@@ -407,22 +398,13 @@ Status MatchExpressionParser::_parseSub(const char* name,
                                         const BSONObj& sub,
                                         AndMatchExpression* root,
                                         const CollatorInterface* collator,
-                                        int level) {
+                                        bool topLevel) {
     // The one exception to {field : {fully contained argument} } is, of course, geo.  Example:
     // sub == { field : {$near[Sphere]: [0,0], $maxDistance: 1000, $minDistance: 10 } }
     // We peek inside of 'sub' to see if it's possibly a $near.  If so, we can't iterate over
     // its subfields and parse them one at a time (there is no $maxDistance without $near), so
     // we hand the entire object over to the geo parsing routines.
 
-    if (level > kMaximumTreeDepth) {
-        mongoutils::str::stream ss;
-        ss << "exceeded maximum query tree depth of " << kMaximumTreeDepth << " at "
-           << sub.toString();
-        return Status(ErrorCodes::BadValue, ss);
-    }
-
-    level++;
-
     // Special case parsing for geoNear. This is necessary in order to support query formats like
     // {$near: <coords>, $maxDistance: <distance>}. No other query operators allow $-prefixed
     // modifiers as sibling BSON elements.
@@ -452,7 +434,9 @@ Status MatchExpressionParser::_parseSub(const char* name,
     while (j.more()) {
         BSONElement deep = j.next();
 
-        StatusWithMatchExpression s = _parseSubField(sub, root, name, deep, collator, level);
+        const bool childIsTopLevel = false;
+        StatusWithMatchExpression s =
+            _parseSubField(sub, root, name, deep, collator, childIsTopLevel);
         if (!s.isOK())
             return s.getStatus();
 
@@ -682,7 +666,7 @@ StatusWithMatchExpression MatchExpressionParser::_parseType(const char* name,
 StatusWithMatchExpression MatchExpressionParser::_parseElemMatch(const char* name,
                                                                  const BSONElement& e,
                                                                  const CollatorInterface* collator,
-                                                                 int level) {
+                                                                 bool topLevel) {
     if (e.type() != Object)
         return {Status(ErrorCodes::BadValue, "$elemMatch needs an Object")};
 
@@ -712,7 +696,7 @@ StatusWithMatchExpression MatchExpressionParser::_parseElemMatch(const char* nam
         // value case
 
         AndMatchExpression theAnd;
-        Status s = _parseSub("", obj, &theAnd, collator, level);
+        Status s = _parseSub("", obj, &theAnd, collator, topLevel);
         if (!s.isOK())
             return s;
 
@@ -736,7 +720,7 @@ StatusWithMatchExpression MatchExpressionParser::_parseElemMatch(const char* nam
 
     // object case
 
-    StatusWithMatchExpression subRaw = _parse(obj, collator, level);
+    StatusWithMatchExpression subRaw = _parse(obj, collator, topLevel);
     if (!subRaw.isOK())
         return subRaw;
     std::unique_ptr<MatchExpression> sub = std::move(subRaw.getValue());
@@ -759,7 +743,7 @@ StatusWithMatchExpression MatchExpressionParser::_parseElemMatch(const char* nam
 StatusWithMatchExpression MatchExpressionParser::_parseAll(const char* name,
                                                            const BSONElement& e,
                                                            const CollatorInterface* collator,
-                                                           int level) {
+                                                           bool topLevel) {
     if (e.type() != Array)
         return {Status(ErrorCodes::BadValue, "$all needs an array")};
 
@@ -787,8 +771,9 @@ StatusWithMatchExpression MatchExpressionParser::_parseAll(const char* name,
                 return {Status(ErrorCodes::BadValue, "$all/$elemMatch has to be consistent")};
             }
 
-            StatusWithMatchExpression inner =
-                _parseElemMatch(name, hopefullyElemMatchObj.firstElement(), collator, level);
+            const bool childIsTopLevel = false;
+            StatusWithMatchExpression inner = _parseElemMatch(
+                name, hopefullyElemMatchObj.firstElement(), collator, childIsTopLevel);
             if (!inner.isOK())
                 return inner;
             myAnd->add(inner.getValue().release());
