@@ -209,11 +209,7 @@ protected:
      */
     class StoredFuncModLogOpHandler;
 
-    virtual FunctionCacheMap& getFunctionCache() {
-        return _cachedFunctions;
-    }
-    virtual ScriptingFunction _createFunction(const char* code,
-                                              ScriptingFunction functionNumber = 0) = 0;
+    virtual ScriptingFunction _createFunction(const char* code) = 0;
 
     std::string _localDBName;
     int64_t _loadedVersion;
