@@ -61,6 +61,7 @@
 #include "mongo/scripting/mozjs/regexp.h"
 #include "mongo/scripting/mozjs/timestamp.h"
 #include "mongo/scripting/mozjs/uri.h"
+#include "mongo/stdx/unordered_set.h"
 
 namespace mongo {
 namespace mozjs {
@@ -148,8 +149,7 @@ public:
 
     void injectNative(const char* field, NativeFunction func, void* data = 0) override;
 
-    ScriptingFunction _createFunction(const char* code,
-                                      ScriptingFunction functionNumber = 0) override;
+    ScriptingFunction _createFunction(const char* code) override;
 
     void newFunction(StringData code, JS::MutableHandleValue out);
 
@@ -297,9 +297,6 @@ public:
         return _uriProto;
     }
 
-    void setQuickExit(int exitCode);
-    bool getQuickExit(int* exitCode);
-
     static const char* const kExecResult;
     static const char* const kInvokeResult;
 
@@ -316,10 +313,35 @@ public:
         return _internedStrings.getInternedString(name);
     }
 
+    std::string buildStackString();
+
+    template <typename T, typename... Args>
+    T* trackedNew(Args&&... args) {
+        T* t = new T(std::forward<Args>(args)...);
+        _asanHandles.addPointer(t);
+        return t;
+    }
+
+    template <typename T>
+    void trackedDelete(T* t) {
+        _asanHandles.removePointer(t);
+        delete (t);
+    }
+
+    struct ASANHandles {
+        ASANHandles();
+        ~ASANHandles();
+
+        void addPointer(void* ptr);
+        void removePointer(void* ptr);
+
+        stdx::unordered_set<void*> _handles;
+
+        static ASANHandles* getThreadASANHandles();
+    };
+
 private:
-    void _MozJSCreateFunction(const char* raw,
-                              ScriptingFunction functionNumber,
-                              JS::MutableHandleValue fun);
+    void _MozJSCreateFunction(StringData raw, JS::MutableHandleValue fun);
 
     /**
      * This structure exists exclusively to construct the runtime and context
@@ -361,6 +383,7 @@ private:
 
     void setCompileOptions(JS::CompileOptions* co);
 
+    ASANHandles _asanHandles;
     MozJSScriptEngine* _engine;
     MozRuntime _mr;
     JSRuntime* _runtime;
@@ -377,8 +400,6 @@ private:
     std::atomic<bool> _pendingGC;
     ConnectState _connectState;
     Status _status;
-    int _exitCode;
-    bool _quickExit;
     std::string _parentStack;
     std::size_t _generation;
     bool _hasOutOfMemoryException;
@@ -416,5 +437,9 @@ inline MozJSImplScope* getScope(JSContext* cx) {
     return static_cast<MozJSImplScope*>(JS_GetContextPrivate(cx));
 }
 
+inline MozJSImplScope* getScope(JSFreeOp* fop) {
+    return static_cast<MozJSImplScope*>(JS_GetRuntimePrivate(fop->runtime()));
+}
+
 }  // namespace mozjs
 }  // namespace mongo
